#!/bin/sh

## Set DISPLAY Parameter
# AUTOMKDISP=0; Display all images
#            1; Display minimum images
#            2; Display No images
if [ `env | grep AUTOMKDISP |gawk 'END{print(NR)}'` -eq 0 ]; then
AUTOMKDISP=0
fi

## Set Debug Parameter 
# AUTOMKDEBUG=1; erase all temporary files 
#             0; leave all temporary files 
if [ `env | grep AUTOMKDEBUG |gawk 'END{print(NR)}'` -eq 0 ]; then
 AUTOMKDEBUG=1
fi
 
## Parameters for Photometry
# Radius within which detect object to do photometry
PHOTRADIUS=200
# Radius of annulus to measure sky
PHOTAN=10
# Width of annulus to measure sky
PHOTDAN=10

if [ $# -ne 4 -a $# -ne 3 ]; then
    echo "Usage : "$0" (FITS data table) (# of dithering point) (Header Name) [(coordinate file)]"
    exit 1
fi

FTABLE=$1
NPOINT=$2
HEADER=`gawk -v hd=$3 'BEGIN{gsub(".fits","",hd);gsub(".imh","",hd);print(hd)}'`
OUTIM=$HEADER".fits"
if [ $# -eq 4 ]; then
 TARGET=$4
else
 TARGET="AUTO"
fi

PHOTTABLE=RESULTS/phot.$HEADER
OFFSETTABLE=RESULTS/offset.$HEADER

MLCOMBTABLE=mlcomb.tbl

NFRAME=`gawk -v np=$NPOINT 'END{print(NR/np)}' $FTABLE`
echo NFRAME=$NFRAME

DITH=20
DITH2=`expr $DITH + $DITH`

TMPFILELIST=tmpfile.list
if [ -f $TMPFILELIST ]; then
  \rm $TMPFILELIST
fi

CLLOG=log/cl`date +%y%m%d%H%M%S`.log
TMPLOG=tmp.log
CLCOM=cl.com

echo $TMPLOG >> $TMPFILELIST
echo $CLCOM >> $TMPFILELIST

if [ ! -d log ]; then
  mkdir log
fi

if [ -f $CLLOG ]; then
  rm $CLLOG
fi

echo "IRAF Log of $0 $1 $2 $3" > $CLLOG

echo "IRAF log will be written in $CLLOG"

echo "### Preparation ###"
echo "### Preparation ###" >> $CLLOG

# Check whether image viewer is running.
cat <<EOF > $CLCOM
 display flat.fits 1
 logout
EOF

cat $CLCOM | cl >& $TMPLOG
cat $TMPLOG >> $CLLOG
ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`
if [ $ERRSTAT -ne 0 ]; then
 echo "# Image Viewer doesn't seem to be running. #"
 echo "# Change AUTOMKDISP to 2 #" 
 AUTOMKDISP=2
fi

#datapars
cat << EOF > tmpdata.par
scale,r,h,1.,0.
fwhmpsf,r,h,10,0.
emission,b,h,yes
sigma,r,h,INDEF
datamin,r,h,INDEF
datamax,r,h,INDEF
noise,s,h,"poisson"
ccdread,s,h,""
gain,s,h,"GAIN"
readnoise,r,h,15
epadu,r,h,1
exposure,s,h,"EXPTIME"
airmass,s,h,"AIRMASS"
filter,s,h,"FILTER01"
obstime,s,h,""
itime,r,h,1.
xairmass,r,h,INDEF
ifilter,s,h,INDEF
otime,s,h,INDEF
EOF

#centerpars
cat <<EOF > tmpcenter.par
calgorithm,s,h,"centroid"
cbox,r,h,5
cthreshold,r,h,0
minsnratio,r,h,1.,0
cmaxiter,i,h,10
maxshift,r,h,1
clean,b,h,no
rclean,r,h,1
rclip,r,h,2.
kclean,r,h,3
mkcenter,b,h,no
EOF

#fitskypars
cat <<EOF > tmpfitsky.par
salgorithm,s,h,"mode"
annulus,r,h,RADAN
dannulus,r,h,$PHOTDAN
skyvalue,r,h,0.
smaxiter,i,h,10
sloclip,r,h,0.
shiclip,r,h,0.
snreject,i,h,50
sloreject,r,h,3.
shireject,r,h,3.
khist,r,h,3.
binsize,r,h,0.1
smooth,b,h,no
rgrow,r,h,0.
mksky,b,h,no
EOF

#photpars
cat << EOF > tmpphot.par
weighting,s,h,"constant"
apertures,s,h,RADAP
zmag,r,h,25.
mkapert,b,h,no
EOF

cat <<EOF >> $TMPFILELIST
tmpphot.par
tmpdata.par
tmpfitsky.par
tmpcenter.par
tmpphot2.par
tmpfitsky2.par
EOF

TMPPHOT=tmp.phot
TMPPHOT2=tmp2.phot
TMPPSF=tmp.psf
TMPPSF2=tmp2.psf
COORDBASE=base.coo
echo $TMPPHOT >> $TMPFILELIST
echo $TMPPHOT2 >> $TMPFILELIST
echo $TMPPSF >> $TMPFILELIST
echo $TMPPSF2 >> $TMPFILELIST
echo $COORDBASE >> $TMPFILELIST

echo "### Photometry ###"
echo "### Photometry ###" >> $CLLOG

if [ $TARGET = "AUTO" ]; then
  echo "# Object Detection #"
  echo "# Object Detection #" >> $CLLOG
  #calculate background sigma
  echo "del tmp.sigma" > $CLCOM
  echo "imstatrej RESULTS/"$OUTIM"[300:600,300:600] 3 > tmp.sigma" >> $CLCOM
  echo "logout" >> $CLCOM

  echo tmp.sigma >> $TMPFILELIST

  cat $CLCOM | cl >& $TMPLOG
  cat $TMPLOG >> $CLLOG
  ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`

  if [ $ERRSTAT -ne 0 ]; then
   echo "# Error in IRAF ... Exiting #"
   echo "# Check $CLLOG. #" 
   exit 1
  fi
  SIGMA=`gawk '{if(NR==1)print($3*4)}' tmp.sigma`

  cat << EOF > $CLCOM
  noao
  digiphot
  apphot
  del tmp.tot.coo 
  imdel tmp1.fits,tmp1.imh
  gauss RESULTS/$OUTIM tmp1 2.5
EOF
  if [ $DITH -gt 0 ];  then
    cat << EOF >> $CLCOM
     imshift tmp1 tmp1 $DITH $DITH boundary=constant
     imshift tmp1 tmp1 -$DITH2 0 boundary=constant 
     imshift tmp1 tmp1 0 -$DITH2 boundary=constant 
     imshift tmp1 tmp1 $DITH $DITH boundary=constant 
EOF
  fi

  cat <<EOF >> $CLCOM
   daofind tmp1 output=tmp.tot.coo fwhmpsf=15 thresho=4 verify- sigma=$SIGMA sharplo=-0.5 sharphi=5 roundlo=-5 roundhi=5 
   imdel tmp1
   logout
EOF

  if [ $AUTOMKDISP -lt 2 ]; then
    cat << EOF >> $CLCOM
     display RESULTS/$OUTIM 1 fi+ 
     tvmark 1 tmp.tot.coo number+
EOF
  fi

  cat $CLCOM | cl >& $TMPLOG
  cat $TMPLOG >> $CLLOG
  ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`

  if [ $ERRSTAT -ne 0 ]; then
   echo "# Error in IRAF ... Exiting #"
   echo "# Check $CLLOG. #" 
   exit 1
  fi
fi

gawk '{x[NR]=$1;y[NR]=$2;if(NR==1){xmin=$1;ymin=$2;}else{ if(xmin>$1) xmin=$1;if(ymin>$2) ymin=$2;}}END{for(i=1;i<=NR;i++) printf("%d %d\n",x[i]-xmin,y[i]-ymin)}'  $OFFSETTABLE > tmp.offset
XCENTER=`gawk '{if(NR==1){xmax=$1;ymax=$2;}else{if(xmax<$1) xmax=$1;if(ymax<$2) ymax=$2;}}END{print(xmax/2+512)}' tmp.offset`
YCENTER=`gawk '{if(NR==1){xmax=$1;ymax=$2;}else{if(xmax<$1) xmax=$1;if(ymax<$2) ymax=$2;}}END{print(ymax/2+512)}' tmp.offset`
echo "# XCENTER=$XCENTER YCENTER=$YCENTER"

echo tmp.offset >> $TMPFILELIST

if [ $TARGET = "AUTO" ]; then 
  grep -v "^#" tmp.tot.coo | gawk -v xc=$XCENTER -v yc=$YCENTER -v rlim=$PHOTRADIUS 'BEGIN{i=1}{x=$1-xc;y=$2-yc;r=sqrt(x*x+y*y);if(r<=rlim){print($1,$2,$3,$4,$5,$6,i);i++}}' > $COORDBASE
else
  gawk '{print($1,$2,"-1 1 -1 -1 "NR)} ' $TARGET > $COORDBASE
fi

NLINE=`gawk 'END{print(NR)}' $COORDBASE `

if [ $NLINE -eq 0 -a $TARGET = "AUTO"  ]; then
  echo " !! Error !!"
  echo "    No Star found in circle(r=$PHOTRADIUS) centered at ($XCENTER $YCENTER)"
  exit 1
fi

if [ $NLINE -eq 0 -a $TARGET != "AUTO" ]; then
  echo " \!\! Error \!\!"
  echo "    No point specified in $TARGET  "
  exit 1
fi

if [ $AUTOMKDISP -lt 2 ]; then
  echo "display RESULTS/"$HEADER".fits 1 fi+" > $CLCOM
  echo "tvmark 1 $COORDBASE number+" >> $CLCOM
  echo "logout" >> $CLCOM

  cat $CLCOM | cl >& $TMPLOG
  cat $TMPLOG >> $CLLOG
  ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`

  if [ $ERRSTAT -ne 0 ]; then
   echo "# Error in IRAF ... Exiting #"
   echo "# Check $CLLOG. #" 
   exit 1
  fi
fi

echo "# Measure following stars :"
cat $COORDBASE

echo "# Aperture Photometry in Process ... #"
echo "# Aperture Photometry in Process ... #" >> $CLLOG

if [ -f $PHOTTABLE ]; then
 \rm $PHOTTABLE
fi

echo "#id itime filter epadu, x y fwhm peak, stddev nsky aperture area flux, stddev nsky aperture area flux ...." >> $PHOTTABLE
i=1

while [ $i -le $NPOINT ]
do
  gawk -v p=$i -v f=$NFRAME '{if(NR>(p-1)*f && NR<=p*f) print $0}' mldata.tbl > mldata.tmp.tbl
  echo mldata.tmp.tbl >> $TMPFILELIST
  XOFFSET=`sed -n ${i}p tmp.offset | gawk '{print($1)}'` 
  YOFFSET=`sed -n ${i}p tmp.offset | gawk '{print($2)}'` 
  COORDFILE=$HEADER.`gawk -v i=$i 'BEGIN{printf("%03d\n",i)}'`.tbl
  gawk -v xo=$XOFFSET -v yo=$YOFFSET '{print($1-xo,$2-yo,$3,$4,$5,$6,$7)}' $COORDBASE > $COORDFILE
  echo $COORDFILE >> $TMPFILELIST
  for filename in `cat mldata.tmp.tbl`
  do
    echo " # Measureing $filename #"
    echo " # Measureing $filename #" >> $CLLOG
    j=1
    NAPERTURE=4
    while [ $j -le $NAPERTURE ]
    do
      APERTURE=`echo "( $j + 1 ) * 5" | bc`
      ANNULUS=`echo $APERTURE + $PHOTAN | bc`

      sed "s/RADAP/$APERTURE/g" tmpphot.par > tmpphot2.par
      sed "s/RADAN/$ANNULUS/g" tmpfitsky.par > tmpfitsky2.par
      if [ -f $TMPPHOT.$j ]; then
        \rm $TMPPHOT.$j
      fi
      cat << EOF > $CLCOM 
       noao
       digiphot
       apphot
EOF
      if [ $j -eq 1 -a $AUTOMKDISP -lt 2 ]; then
        cat <<EOF >> $CLCOM 
	display $filename 1
	tvmark 1 $COORDFILE number+ radii=15 mark=circle
EOF
      fi
      cat <<EOF >> $CLCOM 
       phot $filename coords=$COORDFILE output=$TMPPHOT.$j interac- verify- fitskyp=tmpfitsky2.par photpars=tmpphot2.par datapars=tmpdata.par centerpars=tmpcenter.par
logout
EOF

      cat $CLCOM | cl >& $TMPLOG
      cat $TMPLOG >> $CLLOG
      ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`

      if [ $ERRSTAT -ne 0 ]; then
       echo "# Error in IRAF ... Exiting #"
       echo "# Check $CLLOG. #" 
       exit 1
      fi

      sed '/^#/d' $TMPPHOT.$j | gawk 'BEGIN{i=1}{j=NR%5;if(j==3)printf("%s %s ",$2,$4);if(j==0){print($1,$3,$2);i++}}' > $TMPPHOT2.$j
      echo $TMPPHOT2.$j >> $TMPFILELIST

      j=`expr $j + 1`
    done
    sed '/^#/d' $TMPPHOT.1 | gawk -v conv=3.5 'BEGIN{i=1}{j=NR%5;if(j==1)printf("%d ",i);if(j==4){printf("%s %s %.2f\n",$1,$3,conv);i++}}' > $TMPPHOT2.com
    echo $TMPPHOT2.com >> $TMPFILELIST

    echo "noao" > $CLCOM 
    echo "digiphot" >> $CLCOM 
    echo "apphot" >> $CLCOM
    if [ -f $TMPPSF ]; then
      \rm $TMPPSF
    fi
    echo "fitpsf $filename 20 coords=$COORDFILE output=$TMPPSF interac- verify-" >> $CLCOM
    echo "logout" >> $CLCOM

    cat $CLCOM | cl >& $TMPLOG
    cat $TMPLOG >> $CLLOG
    ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`

    if [ $ERRSTAT -ne 0 ]; then
     echo "# Error in IRAF ... Exiting #"
     echo "# Check $CLLOG. #" 
     exit 1
    fi

    sed '/^#/d' $TMPPSF | gawk 'BEGIN{i=1}{j=NR%3;if(j==2){printf("%s %s %.2f %s\n",$1,$2,2.354820045*$3,$4);i++}}' > $TMPPSF2

    j=1
    NAPERTURE=4
    FILELIST=`echo $TMPPHOT2.com" "$TMPPSF2`
    while [ $j -le $NAPERTURE ] 
    do
      FILELIST=`echo $FILELIST" "$TMPPHOT2.$j`
      j=`expr $j + 1`
    done
    echo "# $filename " >> $PHOTTABLE
    paste $FILELIST >> $PHOTTABLE
  done
  i=`expr $i + 1`
done


COORDFILE=$COORDBASE
filename=RESULTS/$OUTIM
echo " # Measureing $filename #"
echo " # Measureing $filename #" >> $CLLOG
j=1
NAPERTURE=4
while [ $j -le $NAPERTURE ]
do
  APERTURE=`echo "( $j + 1 ) * 5" | bc`
  ANNULUS=`echo $APERTURE + $PHOTAN | bc`
  sed "s/RADAP/$APERTURE/g" tmpphot.par > tmpphot2.par
  sed "s/RADAN/$ANNULUS/g" tmpfitsky.par > tmpfitsky2.par
  if [ -f $TMPPHOT.$j ]; then
    \rm $TMPPHOT.$j
  fi
  cat <<EOF > $CLCOM 
  noao
  digiphot
  apphot
  phot $filename coords=$COORDFILE output=$TMPPHOT.$j interac- verify- fitskyp=tmpfitsky2.par photpars=tmpphot2.par datapars=tmpdata.par centerpars=tmpcenter.par
logout
EOF

  cat $CLCOM | cl >& $TMPLOG
  cat $TMPLOG >> $CLLOG
  ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`

  if [ $ERRSTAT -ne 0 ]; then
   echo "# Error in IRAF ... Exiting #"
   echo "# Check $CLLOG. #" 
   exit 1
  fi

  sed '/^#/d' $TMPPHOT.$j | gawk 'BEGIN{i=1}{j=NR%5;if(j==3)printf("%s %s ",$2,$4);if(j==0){print($1,$3,$2);i++}}' > $TMPPHOT2.$j
  j=`expr $j + 1`
done

sed '/^#/d' $TMPPHOT.1 | gawk -v conv=3.5 'BEGIN{i=1}{j=NR%5;if(j==1)printf("%d ",i);if(j==4){printf("%s %s %.2f\n",$1,$3,conv);i++}}' > $TMPPHOT2.com

cat <<EOF > $CLCOM 
 noao
 digiphot
 apphot
 fitpsf $filename 20 coords=$COORDFILE output=$TMPPSF interac- verify-
 logout
EOF

if [ -f $TMPPSF ]; then
  \rm $TMPPSF
fi

cat $CLCOM | cl >& $TMPLOG
cat $TMPLOG >> $CLLOG
ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`
if [ $ERRSTAT -ne 0 ]; then
 echo "# Error in IRAF ... Exiting #"
 echo "# Check $CLLOG. #" 
 exit 1
fi

sed '/^#/d' $TMPPSF | gawk 'BEGIN{i=1}{j=NR%3;if(j==2){printf("%s %s %.2f %s\n",$1,$2,2.354820045*$3,$4);i++}}' > $TMPPSF2

j=1
NAPERTURE=4
FILELIST=`echo $TMPPHOT2.com" "$TMPPSF2`
while [ $j -le $NAPERTURE ]; do
  FILELIST=`echo $FILELIST" "$TMPPHOT2.$j`
  j=`expr $j + 1`
done
echo "# $filename " >> $PHOTTABLE
paste $FILELIST >> $PHOTTABLE

j=1
while [ $j -le $NAPERTURE ]; do
  echo $TMPPHOT.$j >> $TMPFILELIST
  echo $TMPPHOT2.$j >> $TMPFILELIST
  j=`expr $j + 1`
done

echo "### $0 Finished ###"
echo "### $0 Finished ###" >> $CLLOG
echo "IRAF log written in $CLLOG"
echo "Results of photometry are saved in $PHOTTABLE"
if [ $AUTOMKDEBUG -eq 1 ]; then
  for filename in `cat $TMPFILELIST` ; do
     if [ -f $filename ]; then
      \rm $filename
     fi
  done
fi
