#!/bin/sh

## Set DISPLAY Parameter
# AUTOMKDISP=0; Display all images
#            1; Display minimum images
#            2; Display No images
if [ `env | grep AUTOMKDISP |gawk 'END{print(NR)}'` -eq 0 ]; then
AUTOMKDISP=0
fi

## Set Debug Parameter 
# AUTOMKDEBUG=1; erase all temporary files 
#             0; leave all temporary files 
if [ `env | grep AUTOMKDEBUG |gawk 'END{print(NR)}'` == 0 ]; then
AUTOMKDEBUG=1
fi


## Set Pixel Scale
PIXSCALE=0.105 

if [ $# != 3 ]; then
    echo "Usage : "$0" (FITS data table) (# of dithering point) (Header Name)"
    exit 1
fi

INTABLE=$1
NPOINT=$2
HEADER=`gawk -v hd=$3 'BEGIN{gsub(".fits","",hd);gsub(".imh","",hd);print(hd)}'`

FLATFRAME=./flat.fits
OUTIM=RESULTS/ql.$HEADER".fits"
PHOTTABLE=RESULTS/phot.$HEADER
OFFSETTABLE=RESULTS/offset.$HEADER
MASKTABLE=RESULTS/mask.$HEADER

SKY1TBL=sky1.tbl
SKY2TBL=sky2.tbl
SKYSUBTBL=skysub.tbl
FTABLE=tmp.input.tbl
LDATATBL=ldata.tbl
LDATA1TBL=ldata.1.tbl
LDATA2TBL=ldata.2.tbl
LDATA1TBLREJ=ldatar.1.tbl
LDATA2TBLREJ=ldatar.2.tbl
MLDATATBL=mldata.tbl
PLDATATBL=pldata.tbl
PLDATA1TBL=pldata.1.tbl
PLDATA2TBL=pldata.2.tbl
#file for weight calculation
WEIGHTTBL=tmp.weight.tbl
WEIGHT1TBL=tmp.weight1.tbl
WEIGHT2TBL=tmp.weight2.tbl
# file for photometry
TMPPHOT1=tmp.phot1
TMPPHOT2=tmp.phot2
TMPPSF1=tmp.psf1
TMPPSF2=tmp.psf2
PHOTCOORD=tmp.phot.coo
TMPMASK=tmp.mask

TMPFILELIST=tmpfile.list
GLOBALTMPFILELIST=globaltmpfile.list
GLOBALTMPIMGLIST=globaltmpimg.list

if [ -f $TMPFILELIST ]; then
  \rm $TMPFILELIST
fi

BADPIXMASK=badpix.cl

CLLOG=log/procstd1.$HEADER.`date +%y%m%d%H%M%S`.log 
TMPLOG=tmp.log
CLCOM=cl.com

# Register Local Temporary Files
cat << EOF >> $TMPFILELIST
 $FTABLE
 $TMPPHOT1
 $TMPPHOT2
 $TMPPSF1
 $TMPPSF2
 $TMPLOG
 $PHOTCOORD
 $WEIGHTTBL
 $WEIGHT1TBL
 $WEIGHT2TBL
 $PLDATATBL
 $PLDATA1TBL
 $PLDATA2TBL
 $TMPMASK
 $LDATA1TBLREJ
 $LDATA2TBLREJ
EOF

# Register Global Temporary Files
cat <<EOF >> $GLOBALTMPFILELIST
 $SKY1TBL
 $SKY2TBL
 $SKYSUBTBL
 $LDATATBL
 $LDATA1TBL
 $LDATA2TBL
 $MLDATATBL
EOF

if [ ! -d log ]; then 
  mkdir log 
fi

if [ -f $CLLOG ]; then 
  rm $CLLOG 
fi

echo "IRAF Log of $0 $1 $2 $3" > $CLLOG

echo "IRAF log will be written in $CLLOG" 

# Check Architecture
ARCH=`uname`
uname -a >> $CLLOG

# Check IRAF Version
echo "logout" | cl > $TMPLOG
IRAFVER=`grep Revision $TMPLOG | sed 's/-/ /g' |  gawk '{split($0,a,"Revision");print(a[2])}'| gawk '{split($1,a,".");print(a[1]+a[2]/100+a[3]/1000)}'`
echo "# IRAF Version $IRAFVER "
echo "# IRAF Version $IRAFVER " >> $CLLOG

#Start time
if [ $ARCH == "Linux" ]; then
    SECSTART=`date +%s` 
else 
    SECSTART=`date +"%d %H %M %S"| gawk '{print($1*24*3600+$2*3600+$3*60+$4)}'` 
fi

# split filename and weight table
gawk '{print($1)}' $INTABLE > $FTABLE
#gawk '{if(NF==1)print(1);else print(0)}' $INTABLE > $WEIGHTTBL

# if marker is "-", do not reject that file. (reject only files with "x")
# "-" are only rejected in final coaad done by procstd2.csh 
gawk '{if(NF==1)print(1);else if($2=="-")print(1);else print(0)}' $INTABLE > $WEIGHTTBL

echo "### Preparation ###"
echo "### Preparation ###" >> $CLLOG
DITH=20
DITH2=`expr $DITH + $DITH`

NFRAME=`gawk -v np=$NPOINT 'END{print(NR/np)}' $FTABLE`
echo NFRAME=$NFRAME

# create tables necessary
gawk '{n=split($1,a,"/");print("ldata/"a[n])}' $FTABLE | sed 's/.fits/.imh/g' > $LDATATBL
sed 's/ldata/mldata/g' $LDATATBL > $MLDATATBL

sed 's/.imh/.imh[257:768,257:768]/g' $LDATATBL > $PLDATATBL
gawk -v nf=$NFRAME '{if(i<1) print($0); i++; if(i==nf) i=0;}' $LDATATBL > $LDATA1TBL
gawk -v nf=$NFRAME '{if(i<1) print($0); i++; if(i==nf) i=0;}' $WEIGHTTBL > $WEIGHT1TBL
if [ $NFRAME -gt 1 ]; then
  gawk -v nf=$NFRAME '{if(i>=1) print($0); i++; if(i==nf) i=0;}' $LDATATBL > $LDATA2TBL
  gawk -v nf=$NFRAME '{if(i>=1) print($0); i++; if(i==nf) i=0;}' $WEIGHTTBL > $WEIGHT2TBL
fi
sed 's/.imh/.imh[257:768,257:768]/g' $LDATA1TBL > $PLDATA1TBL

for filename in `cat $FTABLE`; do
  if [ ! -f $filename ]; then
    echo Error: $filename does not exist!
    echo Exiting....
    exit 1
  fi
done

i=1
if [ -f $SKY1TBL ]; then
  \rm $SKY1TBL
fi

if [ -f $SKY2TBL ]; then
  \rm $SKY2TBL
fi

if [ -f $SKYSUBTBL ]; then
  \rm $SKYSUBTBL
fi

while [ $i -le $NPOINT ]; do
  gawk -v i=$i 'BEGIN{printf("sky1.%03d.imh\n",i)}' >> $SKY1TBL
  cat $SKY1TBL >> $GLOBALTMPIMGLIST
  gawk -v i=$i 'BEGIN{printf("skysub.%03d.imh\n",i)}' >> $SKYSUBTBL
  if [ $NFRAME -gt 1 ]; then
    gawk -v i=$i 'BEGIN{printf("sky2.%03d.imh\n",i)}' >> $SKY2TBL
    cat $SKY2TBL >> $GLOBALTMPIMGLIST
  fi
  i=`expr $i + 1`
done

# Check whether image viewer is running.
cat <<EOF > $CLCOM
 display flat.fits 1
 logout
EOF
cat $CLCOM | cl >& $TMPLOG
cat $TMPLOG >> $CLLOG
ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`
if [ $ERRSTAT -ne 0 ]; then
 echo "# Image Viewer doesn't seem to be running. #"
 echo "# Change AUTOMKDISP to 2 #" 
 AUTOMKDISP=2
fi

echo "### FITS Header Analysis ###"
echo "### FITS Header Analysis ###" >> $CLLOG 
if [ -f tmphead.log ]; then
  \rm tmphead.log 
fi

echo "imhead @"$FTABLE" longheader=yes > tmphead.log" > $CLCOM
echo tmphead.log >> $TMPFILELIST
echo "logout" >> $CLCOM
#cat $CLCOM | cl >>& $CLLOG  
cat $CLCOM | cl >& $TMPLOG
cat $TMPLOG >> $CLLOG
ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`

if [ $ERRSTAT -ne 0 ]; then
 echo "# Error in IRAF ... Exiting #"
 echo "# Check $CLLOG. #" 
 exit 1
fi

grep "^SLT-PA" tmphead.log | gawk '{print($3)}' | sed "s/'//g" | sed 's/:/ /g' > tmppa.log 
grep "^RA   " tmphead.log | gawk '{print($3)}' | sed "s/'//g" | sed 's/:/ /g' | gawk '{print($1*3600+$2*60+$3)}' > tmpra.log 
grep "^M2-TYPE" tmphead.log | gawk '{print($3)}' | sed "s/'//g" | sed 's/:/ /g' > tmpm2.log 
#grep "^DEC  " tmphead.log | gawk '{print($3)}' | sed "s/'//g" | sed 's/:/ /g' | gawk '{print(-1*($1*3600+$2*60+$3))}' > tmpdec.log 
grep "^DEC  " tmphead.log | gawk '{print($3)}' | sed "s/'//g" | sed 's/:/ /g' | gawk '{if(gsub("-","-",$1)>0)s=-1;else s=1;print(-1*s*(s*$1*3600+$2*60+$3))}' > tmpdec.log 
RA=`gawk '{s+=$1}END{print(s/NR)}' tmpra.log` 
DEC=`gawk '{s+=$1}END{print(s/NR)}' tmpdec.log` 
PA=`gawk '{s+=$1}END{print(s/NR)}' tmppa.log` 

SMTYPE=`sed -n 1p tmpm2.log`
if [ $SMTYPE = "CS_IR" ]; then
  PIXSCALE=0.105
elif [ $SMTYPE = "NS_OPT" ]; then
  PIXSCALE=0.111
fi

echo "Secondary Mirror Type is $SMTYPE"
echo "PIXSCALE set to $PIXSCALE (arcsec/pix)"

gawk -v ps=$PIXSCALE -v ra=$RA -v dec=$DEC '{print(($1-ra)*15*cos(dec/3600/180*3.14159)/ps)}' tmpra.log > tmpra2.log 
gawk -v ps=$PIXSCALE -v ra=$RA -v dec=$DEC '{print(($1-dec)/ps)}' tmpdec.log > tmpdec2.log 
paste tmpra2.log tmpdec2.log | gawk -v pa=$PA '{print(-1*(cos(pa/180*3.14159)*$1+sin(pa/180*3.14159)*$2),-1*(-sin(pa/180*3.14159)*$1+cos(pa/180*3.14159)*$2))}' > tmpoffset.log 
gawk -v nf=$NFRAME '{s1+=$1;s2+=$2;if(NR%nf==0){printf("%d %d\n",s1/nf,s2/nf);s1=0;s2=0}}' tmpoffset.log | gawk '{if(NR==1){dx=$1;dy=$2}print($1-dx,$2-dy)}'> tmp.offset1

cat <<EOF >> $TMPFILELIST
tmppa.log
tmpm2.log
tmpra.log
tmpdec.log
tmpra2.log
tmpdec2.log
tmpoffset.log
tmp.offset1
EOF

echo "### Offset rejection in Quad 2 ###"
echo "### Offset rejection in Quad 2 ###" >> $CLLOG 
cat << EOF > $CLCOM
 imdel @$LDATATBL
 imcopy @$FTABLE @$LDATATBL
# offsetrej_noflat @$LDATATBL
 logout
EOF

cat $CLCOM | cl >& $TMPLOG
cat $TMPLOG >> $CLLOG
ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`

if [ $ERRSTAT -ne 0 ]; then
 echo "# Error in IRAF ... Exiting #"
 echo "# Check $CLLOG. #" 
 exit 1
fi

echo "### Flat-fielding ###"
echo "### Flat-fielding ###" >> $CLLOG 
cat << EOF > $CLCOM
 imarith @$LDATATBL / $FLATFRAME @$LDATATBL
 logout
EOF
#cat $CLCOM | cl >>& $CLLOG  
cat $CLCOM | cl >& $TMPLOG
cat $TMPLOG >> $CLLOG
ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`

if [ $ERRSTAT -ne 0 ]; then
 echo "# Error in IRAF ... Exiting #"
 echo "# Check $CLLOG. #" 
 exit 1
fi


echo "### Preparation for MKSUBGR command ###"
echo "### Preparation for MKSUBGR command ###" >> $CLLOG 
cat << EOF > $CLCOM 
 del level.tbl
 imstatistics @$PLDATA1TBL field=mean fo- > level.tbl
 logout
EOF
#cat $CLCOM | cl >>& $CLLOG  
cat $CLCOM | cl >& $TMPLOG
cat $TMPLOG >> $CLLOG
ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`

echo level.tbl >> $TMPFILELIST

if [ $ERRSTAT -ne 0 ]; then
 echo "# Error in IRAF ... Exiting #"
 echo "# Check $CLLOG. #" 
 exit 1
fi

gawk '{s+=$1;i++;}END{print(s/i)}' level.tbl > tmp.txt
echo tmp.txt >> $TMPFILELIST
cat << EOF > $CLCOM
 imdel sky1,sky2
 imdel @$SKY1TBL
 imdel @$SKY2TBL
 del bkg1.tbl,bkg2.tbl
 logout
EOF
#cat $CLCOM | cl >>& $CLLOG  
cat $CLCOM | cl >& $TMPLOG
cat $TMPLOG >> $CLLOG
ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`

if [ $ERRSTAT -ne 0 ]; then
 echo "# Error in IRAF ... Exiting #"
 echo "# Check $CLLOG. #" 
 exit 1
fi

paste $WEIGHT1TBL $LDATA1TBL > $LDATA1TBLREJ
echo "### mksubgr ###"
echo "### mksubgr ###" >> $CLLOG 
gawk -v nf=1 -v table=$LDATA1TBLREJ -v d=$AUTOMKDISP 'BEGIN{if(d<2) dispparm=1;else dispparm=0}{printf("mksubgr @%s sky1 1 %d %f 257 768 257 768 bkg1.tbl %d\n",table,nf,$1,dispparm)}' tmp.txt > $CLCOM
echo sky1 >> $GLOBALTMPIMGLIST
if [ $NFRAME -gt 1 ]; then
   paste $WEIGHT2TBL $LDATA2TBL > $LDATA2TBLREJ
   gawk -v nf=$NFRAME -v table=$LDATA2TBLREJ -v d=$AUTOMKDISP 'BEGIN{if(d<2) dispparm=1;else dispparm=0}{printf("mksubgr @%s sky2 1 %d %f 257 768 257 768 bkg2.tbl %d\n",table,nf-1,$1,dispparm)}' tmp.txt >> $CLCOM
   echo sky2 >> $GLOBALTMPIMGLIST
fi

cat <<EOF >> $TMPFILELIST
 bkg1.tbl
 bkg2.tbl
EOF

echo "logout" >> $CLCOM  
#cat $CLCOM | cl >>& $CLLOG  
cat $CLCOM | cl >& $TMPLOG
cat $TMPLOG >> $CLLOG
ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`
 
if [ $ERRSTAT -ne 0 ]; then
 echo "# Error in IRAF ... Exiting #"
 echo "# Check $CLLOG. #" 
 exit 1
fi

echo "### Sky-subtraction ###"
echo "### Sky-subtraction ###" >> $CLLOG 
cat << EOF > $CLCOM
 imdel @$SKYSUBTBL
 logout
EOF
#cat $CLCOM | cl >>& $CLLOG  
cat $CLCOM | cl >& $TMPLOG
cat $TMPLOG >> $CLLOG
ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`
  
if [ $ERRSTAT -ne 0 ]; then
 echo "# Error in IRAF ... Exiting #"
 echo "# Check $CLLOG. #" 
 exit 1
fi

if [ $NFRAME -gt 1 ]; then 
  echo "imarith @${SKY2TBL} - sky2 @${SKYSUBTBL}" > $CLCOM
else
  echo "imarith @${SKY1TBL} - sky1 @${SKYSUBTBL}" > $CLCOM
fi

#if badpixelmask exists, do fixpix
if [ -f ${BADPIXMASK} ]; then
  echo "fixpix @${SKYSUBTBL} ${BADPIXMASK}" >> $CLCOM
fi

echo "logout" >> $CLCOM   

cat $CLCOM | cl >& $TMPLOG
cat $TMPLOG >> $CLLOG
ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`
  
if [ $ERRSTAT -ne 0 ]; then
 echo "# Error in IRAF ... Exiting #"
 echo "# Check $CLLOG. #" 
 exit 1
fi

echo "### Pattern Matching ###"
echo "### Pattern Matching ###" >> $CLLOG 

echo " # Detecting Objects in Each Position #"
echo " # Detecting Objects in Each Position #" >> $CLLOG 

for filename in `cat ${SKYSUBTBL}`; do 
  cat << EOF > $CLCOM
   del tmp.sigma
   imstatrej ${filename}[300:600,300:600] 3 > tmp.sigma
   logout
EOF

  cat $CLCOM | cl >& $TMPLOG
  cat $TMPLOG >> $CLLOG
  ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`
    
  if [ $ERRSTAT -ne 0 ]; then
   echo "# Error in IRAF ... Exiting #"
   echo "# Check $CLLOG. #" 
   exit 1
  fi

  SIGMA=`gawk '{if(NR==1)print($3)}' tmp.sigma` 

  cat << EOF > $CLCOM
   noao
   digiphot
   apphot
EOF
  COORDFILE=`echo $filename | sed 's/imh/coo/g'` 
  if [ -f $COORDFILE ]; then 
    \rm $COORDFILE  
  fi
  cat << EOF >> $CLCOM
   imdel tmpmed
   imcopy $filename tmpmed
EOF
  if [ $DITH -gt 0 ]; then
    cat << EOF >> $CLCOM
     imshift tmpmed tmpmed $DITH $DITH boundary=constant
     imshift tmpmed tmpmed -$DITH2 0 boundary=constant
     imshift tmpmed tmpmed 0 -$DITH2 boundary=constant
     imshift tmpmed tmpmed $DITH $DITH boundary=constant
     median tmpmed tmpmed 5 5
EOF
  else 
    cat << EOF >> $CLCOM
     median $filename tmpmed 5 5
EOF
  fi

  cat << EOF >> $CLCOM
   daofind tmpmed output=$COORDFILE fwhmpsf=8 thresho=4 verify- sigma=$SIGMA sharplo=-0.5 sharphi=5 roundlo=-5 roundhi=5
   imdel tmpmed
EOF

  if [ $AUTOMKDISP -lt 2 ]; then
   cat << EOF >> $CLCOM
    display $filename 1 fi+
    tvmark 1 $COORDFILE number+
EOF
  fi

  echo logout >> $CLCOM

  cat $CLCOM | cl >& $TMPLOG
  cat $TMPLOG >> $CLLOG
  ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`
 
  if [ $ERRSTAT -ne 0 ]; then
    echo "# Error in IRAF ... Exiting #"
    echo "# Check $CLLOG. #" 
    exit 1
  fi
done

echo "# Pattern Matching 1 #"
echo "# Pattern Matching 1 #" >> $CLLOG 
i=1 
for filename in `cat $SKYSUBTBL`; do  
  COORDFILE=`echo $filename | sed 's/imh/coo/g'`  
  COORDFILE2=`echo $filename | sed 's/.imh/2.coo/g'`  
  XOFFSET=`sed -n ${i}p tmp.offset1 | gawk '{print($1)}'`
  YOFFSET=`sed -n ${i}p tmp.offset1 | gawk '{print($2)}'`
  grep -v "^#" $COORDFILE | gawk -v dx=$XOFFSET -v dy=$YOFFSET '{print($1+dx,$2+dy)}' > $COORDFILE2
  i=`expr $i + 1`
done

echo " Executing matchpat"
i=1 
for filename in `cat ${SKYSUBTBL}`; do
  COORDFILE2=`echo $filename | sed 's/.imh/2.coo/g'`  
#  echo $COORDFILE2 $i
  if [ $i -eq 1 ]; then
    basefile=$COORDFILE2
    echo "0 0" > tmp.offset2
    echo tmp.offset2 >> $TMPFILELIST
  else
    ./matchpat $basefile $COORDFILE2 20 >> tmp.offset2
  fi
  i=`expr $i + 1`
done

if [ $AUTOMKDEBUG -eq 1 ]; then
  for filename in `cat $SKYSUBTBL`; do
    COORDFILE2=`echo $filename | sed 's/.imh/2.coo/g'`  
    \rm $COORDFILE2
  done
fi

mv tmp.offset1 tmp.offsettmp

echo tmp.offsettmp >> $TMPFILELIST

paste tmp.offsettmp tmp.offset2 | gawk '{print($1+$3,$2+$4)}' > tmp.offset1

echo "# Pattern Matching 2 #"
echo "# Pattern Matching 2 #" >> $CLLOG
i=1 
for filename in `cat $SKYSUBTBL`; do  
  COORDFILE=`echo $filename | sed 's/imh/coo/g'`  
  COORDFILE3=`echo $filename | sed 's/.imh/3.coo/g'`  
  XOFFSET=`sed -n ${i}p tmp.offset1 | gawk '{print($1)}'`
  YOFFSET=`sed -n ${i}p tmp.offset1 | gawk '{print($2)}'`
  grep -v "^#" $COORDFILE | gawk -v dx=$XOFFSET -v dy=$YOFFSET '{print($1+dx,$2+dy)}' > $COORDFILE3
  if [ $AUTOMKDEBUG -eq 1 ]; then 
    \rm $COORDFILE
  fi
  i=`expr $i + 1`
done

echo " Executing matchpat"
i=1 
for filename in `cat $SKYSUBTBL`; do
  COORDFILE3=`echo $filename | sed 's/.imh/3.coo/g'`  
  if [ $i -eq 1 ]; then
    basefile=$COORDFILE3
    echo "0 0" > tmp.offset2
  else
    ./matchpat $basefile $COORDFILE3 10 >> tmp.offset2
  fi
  i=`expr $i + 1`
done

if [ $AUTOMKDEBUG -eq 1 ]; then 
  for filename in `cat $SKYSUBTBL`; do  
    COORDFILE3=`echo $filename | sed 's/.imh/3.coo/g'`  
    \rm $COORDFILE3
  done
fi

paste tmp.offset1 tmp.offset2 | gawk '{print($1+$3,$2+$4)}' > $OFFSETTABLE

echo "### Shift and Combine ###"
echo "### Shift and Combine ###" >> $CLLOG
cat << EOF > $CLCOM
 imdel $OUTIM
 imcombine @$SKYSUBTBL $OUTIM combine=median reject=sigclip outtype='real' lsigma=2 hsigma=2 offset=$OFFSETTABLE
 logout
EOF
#cat $CLCOM | cl >>& $CLLOG  
cat $CLCOM | cl >& $TMPLOG
cat $TMPLOG >> $CLLOG
ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`
  
if [ $ERRSTAT -ne 0 ]; then
 echo "# Error in IRAF ... Exiting #"
 echo "# Check $CLLOG. #" 
 exit 1
fi

echo "### Object Detection ###"
echo "### Object Detection ###" >> $CLLOG
#calculate background sigma
cat << EOF > $CLCOM
 del tmp.sigma
 imstatrej ${OUTIM}[300:600,300:600] 3 > tmp.sigma
 logout
EOF
#cat $CLCOM | cl >>& $CLLOG  
cat $CLCOM | cl >& $TMPLOG
cat $TMPLOG >> $CLLOG
ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`
 
if [ $ERRSTAT -ne 0 ]; then
 echo "# Error in IRAF ... Exiting #"
 echo "# Check $CLLOG. #" 
 exit 1
fi

SIGMA=`gawk '{if(NR==1)print($3*2)}' tmp.sigma`
if [ $AUTOMKDEBUG -eq 1 ]; then
  \rm tmp.sigma
fi

cat << EOF > $CLCOM
  noao
  digiphot
  apphot
  del tmp.tot.coo
  imdel tmp1.fits,tmp1.imh
  gauss $OUTIM tmp1 2.5
EOF
if [ $DITH -gt 0 ]; then
cat << EOF >> $CLCOM
    imshift tmp1 tmp1 ${DITH} ${DITH} boundary=constant
    imshift tmp1 tmp1 -${DITH2} 0 boundary=constant
    imshift tmp1 tmp1 0 -${DITH2} boundary=constant
    imshift tmp1 tmp1 ${DITH} ${DITH} boundary=constant
EOF
fi

echo tmp.tot.coo >> $TMPFILELIST

cat << EOF >> $CLCOM
  daofind tmp1 output=tmp.tot.coo fwhmpsf=15 thresho=4 verify- sigma=${SIGMA} sharplo=-0.5 sharphi=5 roundlo=-5 roundhi=5
  imdel RESULTS/${HEADER}.sky1.fits
  imcopy sky1.imh RESULTS/${HEADER}.sky1.fits
EOF

if [ $AUTOMKDISP -lt 2 ]; then
  echo display $OUTIM 1 fi+ >> $CLCOM
  echo tvmark 1 tmp.tot.coo number+ >> $CLCOM
fi

if [ $NFRAME -gt 1 ]; then
  echo "imdel RESULTS/$HEADER.sky2.fits" >> $CLCOM
  echo "imcopy sky2.imh RESULTS/$HEADER.sky2.fits" >> $CLCOM
fi
echo "logout" >> $CLCOM   

cat $CLCOM | cl >& $TMPLOG
cat $TMPLOG >> $CLLOG
ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`
  
if [ $ERRSTAT -ne 0 ]; then
 echo "# Error in IRAF ... Exiting #"
 echo "# Check $CLLOG. #" 
 exit 1
fi

echo "### Photometry ###"
echo "### Photometry ###" >> $CLLOG
gawk '{print($1,$2,"-1 1 -1 -1 "NR)}' tmp.tot.coo > $PHOTCOORD

## Define files for photometry
#datapars
cat << EOF > tmpdata.par
 scale,r,h,1.,0.
 fwhmpsf,r,h,10,0.
 emission,b,h,yes
 sigma,r,h,INDEF
 datamin,r,h,INDEF
 datamax,r,h,INDEF
 noise,s,h,"poisson"
 ccdread,s,h,""
 gain,s,h,"GAIN"
 readnoise,r,h,15
 epadu,r,h,1
 exposure,s,h,"EXPTIME"
 airmass,s,h,"AIRMASS"
 filter,s,h,"FILTER01"
 obstime,s,h,""
 itime,r,h,1.
 xairmass,r,h,INDEF
 ifilter,s,h,INDEF
 otime,s,h,INDEF
EOF

#centerpars
cat << EOF > tmpcenter.par
 calgorithm,s,h,"centroid"
 cbox,r,h,5
 cthreshold,r,h,0
 minsnratio,r,h,1.,0
 cmaxiter,i,h,10
 maxshift,r,h,1
 clean,b,h,no
 rclean,r,h,1
 rclip,r,h,2.
 kclean,r,h,3
 mkcenter,b,h,no
EOF

#fitskypars
cat << EOF > tmpfitsky.par
 salgorithm,s,h,"mode"
 annulus,r,h,20
 dannulus,r,h,20
 skyvalue,r,h,0.
 smaxiter,i,h,10
 sloclip,r,h,0.
 shiclip,r,h,0.
 snreject,i,h,50
 sloreject,r,h,3.
 shireject,r,h,3.
 khist,r,h,3.
 binsize,r,h,0.1
 smooth,b,h,no
 rgrow,r,h,0.
 mksky,b,h,no
EOF

#photpars
cat << EOF > tmpphot.par
 weighting,s,h,"constant"
 apertures,s,h,20
 zmag,r,h,25.
 mkapert,b,h,no
EOF

if [ -f $TMPPHOT1 ]; then
    \rm $TMPPHOT1
fi
cat << EOF > $CLCOM
  noao
  digiphot
  apphot
  phot tmp1 coords=$PHOTCOORD output=$TMPPHOT1 interac- verify- fitskyp=tmpfitsky.par photpars=tmpphot.par datapars=tmpdata.par centerpars=tmpcenter.par
  logout
EOF

cat $CLCOM | cl >& $TMPLOG
cat $TMPLOG >> $CLLOG
ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`
  
if [ $ERRSTAT -ne 0 ]; then
 echo "# Error in IRAF ... Exiting #"
 echo "# Check $CLLOG. #" 
 exit 1
fi

\rm tmpfitsky.par tmpphot.par tmpdata.par tmpcenter.par
#sed '/^#/d' $TMPPHOT1 | gawk 'BEGIN{i=1}{j=NR%5;if(j==0){print($4);i++}}' > $TMPPHOT2
sed '/^#/d' $TMPPHOT1 |  gawk 'BEGIN{i=1}{j=NR%5;if(j==0){print($4);i++}else if(j==1){printf("%s %s ",$2,$3)}}' > $TMPPHOT2

echo "### Measure FWHM ###"
echo "### Measure FWHM ###" >> $CLLOG

if [ -f $TMPPSF1 ]; then
 \rm $TMPPSF1
fi

cat << EOF > $CLCOM
  noao
  digiphot
  apphot
  fitpsf tmp1 50 coords=$PHOTCOORD output=$TMPPSF1 interac- verify-
  imdel tmp1
  logout
EOF
#cat $CLCOM | cl >>& $CLLOG  
cat $CLCOM | cl >& $TMPLOG
cat $TMPLOG >> $CLLOG
ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`
  
if [ $ERRSTAT -ne 0 ]; then
 echo "# Error in IRAF ... Exiting #"
 echo "# Check $CLLOG. #" 
 exit 1
fi

sed '/^#/d' $TMPPSF1 | gawk 'BEGIN{i=1}{j=NR%3;if(j==2){printf("%s %s %.2f %s\n",$1,$2,2.354820045*$3,$4);i++}}' > $TMPPSF2
# x y FWHM AMPLITUDE x y Flux
# Use the latter x y. (calculated from phot)
#paste $TMPPSF2 $TMPPHOT2 | gawk '{f=sqrt($5*$5);size=log(f+1e5)*60/log(10)-290;print($1,$2,size-(size/10)%1*10,size-5)}'|cat -n > $MASKTABLE
paste $TMPPSF2 $TMPPHOT2 | gawk '{f=sqrt($7*$7);size=log(f+1e5)*60/log(10)-290;print($5,$6,size-(size/10)%1*10,size-5)}'|cat -n > $MASKTABLE

echo "### $0 Finished ###"
echo "### $0 Finished ###" >> $CLLOG
if [ $AUTOMKDEBUG -eq 1 ]; then 
  echo "imdel @$SKYSUBTBL" > $CLCOM
  echo "logout" >> $CLCOM    
  #cat $CLCOM | cl >>& $CLLOG  
  cat $CLCOM | cl >& $TMPLOG
  cat $TMPLOG >> $CLLOG
  ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`
   
  if [ $ERRSTAT -ne 0 ]; then
   echo "# Error in IRAF ... Exiting #"
   echo "# Check $CLLOG. #" 
   exit 1
  fi
  \rm $SKYSUBTBL
fi

echo $NPOINT" points dithering."
echo "Output Image is "$OUTIM
echo "Sky Image is "RESULTS/$HEADER.sky1.fits
if [ $NFRAME -gt 1 ]; then
  echo "             "RESULTS/$HEADER.sky2.fits
fi

echo "Offset pattern file is saved as "$OFFSETTABLE
echo "Offset pattern is : "
cat $OFFSETTABLE
echo "Sigma value for daofind is "$SIGMA
echo "Mask of objcts is saved as "$MASKTABLE
echo "IRAF log was written in $CLLOG"

if [ $AUTOMKDEBUG -eq 1 ]; then
echo "### Erase Temporary Files ###"
echo "### Erase Temporary Files ###" >> $CLLOG
  for filename in `cat $TMPFILELIST`; do
    if [ -f $filename ]; then
      \rm $filename
    fi
  done
  \rm $TMPFILELIST
fi

#END time
if [ $ARCH == "Linux" ] ; then
    SECEND=`date +%s` 
else
    SECEND=`date +"%d %H %M %S"| gawk '{print($1*24*3600+$2*3600+$3*60+$4)}'` 
fi

echo $SECSTART $SECEND | gawk '{printf("Took %d sec to complete\n",$2-$1)}' >> $CLLOG
echo $SECSTART $SECEND | gawk '{printf("Took %d sec to complete\n",$2-$1)}'
