#!/bin/sh

## Set DISPLAY Parameter
# AUTOMKDISP=0; Display all images
#            1; Display minimum images
#            2; Display No images
if [ `env | grep AUTOMKDISP |gawk 'END{print(NR)}'` -eq 0 ]; then
AUTOMKDISP=0
fi

## Set Debug Parameter 
# AUTOMKDEBUG=1; erase all temporary files 
#             0; leave all temporary files 
if [ `env | grep AUTOMKDEBUG |gawk 'END{print(NR)}'` -eq 0 ]; then
AUTOMKDEBUG=1
fi

## Set Fringe Rejection Parameter
# AUTOMKFRINGE=1; Attempt to Delete FRINGE pattern
#              0; Do nothing
if [ `env | grep AUTOMKFRINGE |gawk 'END{print(NR)}'` -eq 0 ]; then
AUTOMKFRINGE=0
fi

## Set Pixel Scale (Not Used)
#PIXSCALE=0.105  

## Parameters for Photometry
# Radius within which detect object to do photometry
PHOTRADIUS=100
# Radius of annulus to measure sky
PHOTAN=10
# Width of annulus to measure sky
PHOTDAN=10

if [ $# -ne 3 ]; then
    echo "Usage : "$0" (FITS data table) (# of dithering point) (Header Name)"
    exit 1
fi

INTABLE=$1
NPOINT=$2
HEADER=`gawk -v hd=$3 'BEGIN{gsub(".fits","",hd);gsub(".imh","",hd);print(hd)}'`
OUTIM=$HEADER".fits"

MLDATATBL=mldata.tbl
MLCOMBTBL=mlcomb.tbl
FTABLE=tmp.input.tbl 

LDATATBL=ldata.tbl
LDATATBLREJ=ldatar.tbl
LDATATMPTBL=ldata.tmp.tbl
MLDATATMPTBL=mod_ldata.tmp.tbl
LDATA1TBL=ldata.1.tbl
LDATA2TBL=ldata.2.tbl
SKY1TBL=sky1.tbl
SKY2TBL=sky2.tbl
MSKY1TBL=msky1.tbl
MSKY2TBL=msky2.tbl

SKY1TBLFRST=sky1frst.tbl
SKY2TBLFRST=sky2frst.tbl
SKY1TBLLAST=sky1last.tbl
SKY2TBLLAST=sky2last.tbl

WEIGHTTBL=tmp.weight.tbl 
WEIGHT1TBL=tmp.weight1.tbl 
WEIGHT2TBL=tmp.weight2.tbl 

PHOTTABLE=RESULTS/phot.$HEADER
OFFSETTABLE=RESULTS/offset.$HEADER

MASKTBL=tmp.mask
MASKTABLE=RESULTS/mask.$HEADER
TMPMASK=tmpoffmask
TMPMASKFILE=tmpmask.tbl
BASEIMG=tmpzero.imh

#FRINGEFILE=fringe.imh
#FRINGEOUTIM=$HEADER"fringe.fits"
FRINGEFILE=RESULTS/fringe.$HEADER.fits
FRINGETMP1=tmpfringe1.imh

TMPFILELIST=tmpfile.list
TMPIMGLIST=tmpimg.list
GLOBALTMPFILELIST=globaltmpfile.list
GLOBALTMPIMGLIST=globaltmpimg.list

if [ -f $TMPFILELIST ]; then
  \rm $TMPFILELIST
fi
if [ -f $TMPIMGLIST ]; then
  \rm $TMPIMGLIST
fi

BADPIXMASK=badpix.pl

CLLOG=log/procstd2.$HEADER.`date +%y%m%d%H%M%S`.log
TMPLOG=tmp.log
CLCOM=cl.com

echo $FTABLE >> $TMPFILELIST
echo $TMPLOG >> $TMPFILELIST

# Register Local Temporary Files
cat <<EOF >> $TMPFILELIST
 $WEIGHTTBL
 $WEIGHT1TBL
 $WEIGHT2TBL
 $TMPMASK
 $TMPMASKFILE
 $TMPLOG
 $CLCOM
 $LDATATMPTBL
 $MLDATATMPTBL
 $MASKTBL
EOF

# Register Global Temporary Files
cat <<EOF >> $GLOBALTMPFILELIST
 $MLDATATBL
 $MLCOMBTBL
 $LDATATBL
 $LDATATBL
 $LDATATBLREJ
 $LDATA1TBL
 $LDATA2TBL
 $SKY1TBL
 $SKY2TBL
 $MSKY1TBL
 $MSKY2TBL
 $SKY1TBLFRST
 $SKY2TBLFRST
 $SKY1TBLLAST
 $SKY2TBLLAST
EOF

if [ ! -d log ]; then
  mkdir log
fi
if [ -f $CLLOG ]; then
  rm $CLLOG
fi

echo "IRAF Log of $0 $1 $2 $3" > $CLLOG

echo "IRAF log will be written in $CLLOG"

# Check Architecture
ARCH=`uname`
uname -a >> $CLLOG

# Check IRAF Version
echo "logout" | cl > $TMPLOG
IRAFVER=`grep Revision $TMPLOG | sed 's/-/ /g' |  gawk '{split($0,a,"Revision");print(a[2])}'| gawk '{split($1,a,".");print(a[1]+a[2]/100+a[3]/1000)}'`
echo "# IRAF Version $IRAFVER "
echo "# IRAF Version $IRAFVER " >> $CLLOG

#Start time
if [ $ARCH = "Linux" ]; then
    SECSTART=`date +%s` 
else
    SECSTART=`date +"%d %H %M %S"| gawk '{print($1*24*3600+$2*3600+$3*60+$4)}'` 
fi

echo "### Preparation ###"
echo "### Preparation ###" >> $CLLOG
# split filename and weight table
gawk '{print($1)}' $INTABLE > $FTABLE
gawk '{if(NF==1)print(1);else print(0)}' $INTABLE > $WEIGHTTBL
DITH=20
DITH2=`expr $DITH + $DITH`
NFRAME=`gawk -v np=$NPOINT 'END{print(NR/np)}' $FTABLE`
echo NFRAME=$NFRAME

# Check whether image viewer is running.
cat <<EOF > $CLCOM
 display flat.fits 1
 logout
EOF

cat $CLCOM | cl >& $TMPLOG
cat $TMPLOG >> $CLLOG
ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`
if [ $ERRSTAT -ne 0 ]; then
 echo "# Image Viewer doesn't seem to be running. #"
 echo "# Change AUTOMKDISP to 2 #" 
 AUTOMKDISP=2
fi

echo "### Create Offset Masks ###"
echo "### Create Offset Masks ###" >> $CLLOG
gawk '{if($4<=10)print($2,$3)}' $MASKTABLE > $TMPMASK.10
gawk '{if($4==20)print($2,$3)}' $MASKTABLE > $TMPMASK.20
gawk '{if($4==30)print($2,$3)}' $MASKTABLE > $TMPMASK.30
gawk '{if($4==40)print($2,$3)}' $MASKTABLE > $TMPMASK.40
gawk '{if($4==50)print($2,$3)}' $MASKTABLE > $TMPMASK.50
gawk '{if($4>=60)print($2,$3)}' $MASKTABLE > $TMPMASK.60

cat << EOF >> $TMPFILELIST
 $TMPMASK.10 
 $TMPMASK.20 
 $TMPMASK.30 
 $TMPMASK.40 
 $TMPMASK.50 
 $TMPMASK.60 
EOF

\rm MASK/$HEADER.??.0??.tbl >& /dev/null
./offsettbl $TMPMASK.10 $OFFSETTABLE MASK/$TMPMASK.10
./offsettbl $TMPMASK.20 $OFFSETTABLE MASK/$TMPMASK.20
./offsettbl $TMPMASK.30 $OFFSETTABLE MASK/$TMPMASK.30
./offsettbl $TMPMASK.40 $OFFSETTABLE MASK/$TMPMASK.40
./offsettbl $TMPMASK.50 $OFFSETTABLE MASK/$TMPMASK.50
./offsettbl $TMPMASK.60 $OFFSETTABLE MASK/$TMPMASK.60

cat << EOF >> $TMPFILELIST
 MASK/$TMPMASK.10.???.tbl 
 MASK/$TMPMASK.20.???.tbl 
 MASK/$TMPMASK.30.???.tbl 
 MASK/$TMPMASK.40.???.tbl 
 MASK/$TMPMASK.50.???.tbl 
 MASK/$TMPMASK.60.???.tbl 
EOF

echo "### Create Object Mask ###"
echo "### Create Object Mask ###" >> $CLLOG

sed 's/sky1/msky1/g' $SKY1TBL | sed 's/.imh/.pl/g'> $MSKY1TBL
cat $MSKY1TBL >>  $GLOBALTMPIMGLIST
if [ $NFRAME -gt 1 ]; then
  sed 's/sky2/msky2/g' $SKY2TBL | sed 's/.imh/.pl/g' > $MSKY2TBL
  cat $MSKY2TBL >>  $GLOBALTMPIMGLIST
fi


cat << EOF > $CLCOM
 imdel @$MLDATATBL
 imdel @$MSKY1TBL
 imdel @$MSKY2TBL
 imdel $BASEIMG  
 imarith flat.fits * 0 $BASEIMG
 logout
EOF

cat $CLCOM | cl >& $TMPLOG
cat $TMPLOG >> $CLLOG
ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`

if [ $ERRSTAT -ne 0 ]; then
 echo "# Error in IRAF ... Exiting #"
 echo "# Check $CLLOG. #" 
 exit 1
fi

i=1
while [ $i -le $NPOINT ]; do
  echo "# Create Object Mask in Position $i #"
  echo "# Create Object Mask in Position $i #">> $CLLOG
  filename1=`sed -n ${i}p $SKY1TBL`
  filename2=`sed -n ${i}p $MSKY1TBL`
  POINTNUM=`gawk -v n=$i 'BEGIN{printf("%03d\n",n)}'`
  if [ -f $MASKTBL ]; then 
    \rm $MASKTBL
  fi

  # reject points out of frame
  for filename in MASK/$TMPMASK.??.$POINTNUM.tbl; do
    gawk '{if($1<1034 && $1>-10 && $2<1034 && $2>-10) print($0)}' $filename > $TMPMASKFILE
    cat $TMPMASKFILE > $filename
  done
   
  # make list of mask table files
  ls MASK/$TMPMASK.??.$POINTNUM.tbl > $MASKTBL

  echo "createmask $BASEIMG $filename2 @$MASKTBL" > $CLCOM

# Register mask to header
  echo "hedit $filename1 bpm $filename2 add+ ver-" >> $CLCOM
  if [ $NFRAME -gt 1 ]; then
    filename1=`sed -n ${i}p $SKY2TBL`
    filename2=`sed -n ${i}p $MSKY2TBL`
    echo "createmask $BASEIMG $filename2 @$MASKTBL" >> $CLCOM
# Register mask to header
    echo "hedit $filename1 bpm $filename2 add+ ver-" >> $CLCOM
  fi

  echo "logout" >> $CLCOM 

  cat $CLCOM | cl >& $TMPLOG
  cat $TMPLOG >> $CLLOG
  ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`

  if [ $ERRSTAT -ne 0 ]; then
   echo "# Error in IRAF ... Exiting #"
   echo "# Check $CLLOG. #" 
   exit 1
  fi
  i=`expr $i + 1`
done
echo $LDATATMPTBL >> $TMPFILELIST
echo $MLDATATMPTBL >> $TMPFILELIST

echo "### Re-Create Sky Frames ###"
echo "### Re-Create Sky Frames ###" >> $CLLOG
echo "imdel noobjsky1,noobjsky2" > $CLCOM
echo "imdel noobjsky1frst,noobjsky2frst" >> $CLCOM
echo "imdel noobjsky1last,noobjsky2last" >> $CLCOM
echo "imdel $BASEIMG" >> $CLCOM
echo "logout" >> $CLCOM

cat $CLCOM | cl >& $TMPLOG
cat $TMPLOG >> $CLLOG
ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`
if [ $ERRSTAT -ne 0 ]; then
 echo "# Error in IRAF ... Exiting #"
 echo "# Check $CLLOG. #" 
 exit 1
fi

echo "# Combine No-Obj Sky #"
echo "# Combine No-Obj Sky #">> $CLLOG

#create total sky
echo "imcombine @$SKY1TBL noobjsky1 combine=median reject=sigclip outtype=real lsigma=2 hsigma=2 masktype=goodvalue maskvalue=0" > $CLCOM
echo noobjsky1 >> $GLOBALTMPIMGLIST
if [ $NFRAME -gt 1 ]; then 
  echo "imcombine @$SKY2TBL noobjsky2 combine=median reject=sigclip outtype=real lsigma=2 hsigma=2 masktype=goodvalue maskvalue=0" >> $CLCOM
  echo noobjsky2 >> $GLOBALTMPIMGLIST
fi
echo "logout" >> $CLCOM 


echo "logout" >> $CLCOM 
cat $CLCOM | cl >& $TMPLOG
cat $TMPLOG >> $CLLOG
ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`

if [ $ERRSTAT -ne 0 ]; then
 echo "# Error in IRAF ... Exiting #"
 echo "# Check $CLLOG. #" 
 exit 1
fi

#create sky with first half and 2nd half
#Then create a Fringe template file
if [ $NPOINT -gt 5 ]; then
    HALFPOINT=`gawk -v n=$NPOINT 'BEGIN{n=int(n/2);print(n)}'`
    echo "### Create a Fringe Pattern File ###"
    echo "### Create a Fringe Pattern File ###" >> $CLLOG
    gawk -v ls=1 -v l=$HALFPOINT '{le=ls+l-1;if(NR>=ls && NR<=le)print($0)}' $SKY1TBL > $SKY1TBLFRST
    gawk -v le=$NPOINT -v l=$HALFPOINT '{ls=le-l+1;if(NR>=ls && NR<=le)print($0)}' $SKY1TBL > $SKY1TBLLAST
    cat << EOF > $CLCOM
    imcombine @$SKY1TBLFRST noobjsky1frst combine=median reject=sigclip outtype=real lsigma=2 hsigma=2 masktype=goodvalue maskvalue=0
    imcombine @$SKY1TBLLAST noobjsky1last combine=median reject=sigclip outtype=real lsigma=2 hsigma=2 masktype=goodvalue maskvalue=0
    imdel $FRINGETMP1
    imarith noobjsky1frst - noobjsky1last $FRINGETMP1
EOF
fi

if [ $NPOINT -gt 5 -a $NFRAME -gt 1 ]; then
    gawk -v ls=1 -v l=$HALFPOINT '{le=ls+l-1;if(NR>=ls && NR<=le)print($0)}' $SKY2TBL > $SKY2TBLFRST
    gawk -v le=$NPOINT -v l=$HALFPOINT '{ls=le-l+1;if(NR>=ls && NR<=le)print($0)}' $SKY2TBL > $SKY2TBLLAST
    cat << EOF >> $CLCOM
    imcombine @$SKY2TBLFRST noobjsky2frst combine=median reject=sigclip outtype=real lsigma=2 hsigma=2 masktype=goodvalue maskvalue=0
    imcombine @$SKY2TBLLAST noobjsky2last combine=median reject=sigclip outtype=real lsigma=2 hsigma=2 masktype=goodvalue maskvalue=0
    imdel $FRINGETMP1
    imarith noobjsky2frst - noobjsky2last $FRINGETMP1
EOF
fi
if [ -f $BADPIXMASK -a $NPOINT -gt 5 ]; then
    echo "fixpix $FRINGETMP1 $BADPIXMASK" >> $CLCOM
fi

if [ $NPOINT -gt 5 ]; then
  echo "" > tmpfringe.mask
  echo tmpfringe.mask >> $TMPFILELIST
  NPOINTDIFF=`gawk -v a=$NPOINT -v b=$HALFPOINT 'BEGIN{print(a-b)}'`
  cat <<EOF >> $CLCOM 
    imdel $FRINGEFILE
    offsetrej $FRINGETMP1 1
    imshift $FRINGETMP1 $FRINGETMP1 -5 -5 boundar=constant
    imshift $FRINGETMP1 $FRINGETMP1 10 10 boundar=constant
    imshift $FRINGETMP1 $FRINGETMP1 -5 -5 boundar=constant
    modify $FRINGETMP1 $FRINGEFILE @tmpfringe.mask 0
    median $FRINGEFILE $FRINGEFILE 5 5
    boxcar $FRINGEFILE $FRINGEFILE 3 3
    imarith $FRINGEFILE / $NPOINTDIFF $FRINGEFILE
EOF

  if [ $AUTOMKDISP -lt 2 ]; then
    echo display $FRINGEFILE 1 fi+ >> $CLCOM 
  fi

  echo logout >> $CLCOM 

  cat $CLCOM | cl >& $TMPLOG
  cat $TMPLOG >> $CLLOG
  ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`

  if [ $ERRSTAT -ne 0 ]; then
   echo "# Error in IRAF ... Exiting #"
   echo "# Check $CLLOG. #" 
   exit 1
  fi
fi

echo "### Subtract New Sky Frames ###"
echo "### Subtract New Sky Frames ###" >> $CLLOG
echo "imarith @$LDATA1TBL - noobjsky1 @$LDATA1TBL" > $CLCOM
if [ $NFRAME -gt 1 ]; then 
 echo "imarith @$LDATA2TBL - noobjsky2 @$LDATA2TBL" >> $CLCOM
fi
echo "logout" >> $CLCOM

cat $CLCOM | cl >& $TMPLOG
cat $TMPLOG >> $CLLOG
ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`

if [ $ERRSTAT -ne 0 ]; then
 echo "# Error in IRAF ... Exiting #"
 echo "# Check $CLLOG. #" 
 exit 1
fi

echo "### Modify ###"
echo "### Modify ###" >> $CLLOG
echo "# First, reject offset in quad 2 #"
echo "# First, reject offset in quad 2 #" >> $CLLOG
echo "imdel @$MLDATATBL" > $CLCOM
echo "offsetrej_noflat @$LDATATBL" >> $CLCOM

#if badpixelmask exists, do fixpix
if [ -f $BADPIXMASK ]; then
    echo "# Fix bad pixels using ${BADPIXMASK} #"
    echo "# Fix bad pixels using ${BADPIXMASK} #" >> $CLLOG
    for filename in `cat $LDATATBL`; do
	echo "fixpix ${filename} ${BADPIXMASK}" >> $CLCOM
    done
fi

echo "logout" >> $CLCOM 

cat $CLCOM | cl >& $TMPLOG
cat $TMPLOG >> $CLLOG
ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`

if [ $ERRSTAT -ne 0 ]; then
 echo "# Error in IRAF ... Exiting #"
 echo "# Check $CLLOG. #" 
 exit 1
fi

i=1
while [ $i -le $NPOINT ]; do
  echo "# Adjusting Bias Offset in Position $i #"  
  echo "# Adjusting Bias Offset in Position $i #" >> $CLLOG
  gawk -v p=$i -v f=$NFRAME '{if(NR>(p-1)*f && NR<=p*f) print $0}' $LDATATBL > $LDATATMPTBL
  sed 's/ldata/mldata/g' $LDATATMPTBL > $MLDATATMPTBL
  POINTNUM=`gawk -v n=$i 'BEGIN{printf("%03d\n",n)}'`

#scale fringe pattern according to the position, and subtract
    
  \rm $CLCOM  
  if [ $AUTOMKFRINGE -eq 1 -a $NPOINT -gt 5 ]; then
    SCALE=`gawk -v n=$NPOINT -v i=$i 'BEGIN{print(n/2-i)}'`
    cat <<EOF >> $CLCOM
    imdel $FRINGETMP1
    imarith $FRINGEFILE * $SCALE $FRINGETMP1
    imarith @$LDATATMPTBL - $FRINGETMP1 @$MLDATATMPTBL
EOF
  else
    echo "imcopy @$LDATATMPTBL @$MLDATATMPTBL" >> $CLCOM
  fi

  if [ -f $MASKTBL ]; then 
    \rm $MASKTBL
  fi

  for filename in MASK/$TMPMASK.??.$POINTNUM.tbl; do
    if [ -s $filename ]; then
     echo $filename >> $MASKTBL
    fi
  done
  if [ $AUTOMKDISP -eq 0 ]; then
     MODIFYDISP=1
  else
     MODIFYDISP=0
  fi

  echo "modify @$MLDATATMPTBL @$MLDATATMPTBL @$MASKTBL $MODIFYDISP" >> $CLCOM
  echo "logout" >> $CLCOM 

  cat $CLCOM | cl >& $TMPLOG
  cat $TMPLOG >> $CLLOG
  ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`

  if [ $ERRSTAT -ne 0 ]; then
   echo "# Error in IRAF ... Exiting #"
   echo "# Check $CLLOG. #" 
   exit 1
  fi
  i=`expr $i + 1`
done

echo "### Shift and Combine All Frames ###"
echo "### Shift and Combine All Frames ###" >> $CLLOG

echo "# Calculate Sigma for Single Frame #"
echo "# Calculate Sigma for Single Frame #" >> $CLLOG
echo "del tmp.sigma" > $CLCOM 
echo "imstatrej `sed -n 1p $MLDATATBL`[300:600,300:600] 3 > tmp.sigma" >> $CLCOM
echo "logout" >> $CLCOM

cat $CLCOM | cl >& $TMPLOG
cat $TMPLOG >> $CLLOG
ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`

if [ $ERRSTAT -ne 0 ]; then
 echo "# Error in IRAF ... Exiting #"
 echo "# Check $CLLOG. #" 
 exit 1
fi
SIGMA=`gawk '{if(NR==1)print($3*1.5)}' tmp.sigma` 

echo "  sigma=$SIGMA"
echo tmp.sigma >> $TMPFILELIST

echo $MLCOMBTBL >> $TMPFILELIST
if [ -f $MLCOMBTBL ]; then
  \rm $MLCOMBTBL
fi

i=1
while [ $i -le $NPOINT ]; do
  paste $WEIGHTTBL $LDATATBL > $LDATATBLREJ
  echo "# Shift and Add at Position $i #"
  echo "# Shift and Add at Position $i #" >> $CLLOG
  echo " # Object Detection #"
  echo " # Object Detection #" >> $CLLOG
  PMLTABLE=mldata.$i.tbl
  PWEIGHTTBL=weight.$i.tbl
  echo $PMLTABLE >> $TMPFILELIST
  echo $PWEIGHTTBL >> $TMPFILELIST
  PMLCOMB=RESULTS/$HEADER.$i.fits
  PMLCOMBMASK=RESULTS/mask.$HEADER.$i.pl
  paste $WEIGHTTBL $MLDATATBL | gawk -v i=$i -v nf=$NFRAME '{if(NR>nf*(i-1) && NR<=nf*i && $1>0) print($2)}' > $PMLTABLE 
  paste $WEIGHTTBL $MLDATATBL | gawk -v i=$i -v nf=$NFRAME '{if(NR>nf*(i-1) && NR<=nf*i && $1>0) print($1)}' > $PWEIGHTTBL
  echo "noao" > $CLCOM 
  echo "digiphot" >> $CLCOM 
  echo "apphot" >> $CLCOM
  for filename in `cat $PMLTABLE`; do
    COORDFILE=`echo $filename | sed 's/.imh/.coo/g' | sed 's/.fits/.coo/g'`
    echo $COORDFILE >> $GLOBALTMPFILELIST
    if [ -f $COORDFILE ]; then
      \rm $COORDFILE 
    fi
    cat << EOF >> $CLCOM
     imdel tmpmed
     median $filename tmpmed 5 5
     imshift tmpmed tmpmed -20 -20 boundar=constant
     imshift tmpmed tmpmed 40 40 boundar=constant
     imshift tmpmed tmpmed -20 -20 boundar=constant
     daofind tmpmed output=$COORDFILE fwhmpsf=8 thresho=4 verify- sigma=$SIGMA sharplo=-0.5 sharphi=5 roundlo=-5 roundhi=5
     imdel tmpmed
EOF

    if [ $AUTOMKDISP -lt 1 ]; then
	cat << EOF >> $CLCOM
	display $filename 1 fi+
	tvmark 1 $COORDFILE number+
EOF
    fi
  done
  echo "logout" >> $CLCOM

  cat $CLCOM | cl >& $TMPLOG
  cat $TMPLOG >> $CLLOG
  ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`

  if [ $ERRSTAT -ne 0 ]; then
   echo "# Error in IRAF ... Exiting #"
   echo "# Check $CLLOG. #" 
   exit 1
  fi
  \cp $COORDFILE tmp.coord
  grep -v "^#" tmp.coord > $COORDFILE
  \rm tmp.coord
  
  echo " # Pattern Matching #"
  echo " # Pattern Matching #" >> $CLLOG
  j=1 
  for filename in `cat $PMLTABLE`; do  
    COORDFILE=`echo $filename | sed 's/.imh/.coo/g'| sed 's/.fits/.coo/g'`  
#    echo $COORDFILE $j 
    echo $COORDFILE >> $GLOBALTMPFILELIST
    if [ $j -eq 1 ]; then 
      BASEFILE=$COORDFILE
      echo "0 0" > tmp.offset 
    else
      (./matchpat $BASEFILE $COORDFILE 20 >> tmp.offset) >& $TMPLOG
      cat $TMPLOG
      cat $TMPLOG >> $CLLOG
    fi

    j=`expr $j + 1`
  done
  echo " # Combine frames: Output is $PMLCOMB #"
  echo " #               : Mask   is $PMLCOMBMASK #"
  echo " # Combine frames: Output is $PMLCOMB #" >> $CLLOG
  echo " #               : Mask   is $PMLCOMBMASK #" >> $CLLOG

  \rm $CLCOM
# Register Bad-pixel Mask
  for filename in `cat $PMLTABLE`; do  
    if [ -f $BADPIXMASK ]; then
      echo "hedit $filename bpm ${BADPIXMASK} add+ ver-" >> $CLCOM
    fi
  done

  cat << EOF >> $CLCOM
   imdel $PMLCOMB
   imdel $PMLCOMBMASK
   imcombine @$PMLTABLE $PMLCOMB rejmask=$PMLCOMBMASK combine=average reject=sigclip outtype=real lsigma=3 hsigma=3 offset=tmp.offset weight=@$PWEIGHTTBL masktype=goodvalue maskvalue=0
EOF
  echo "hedit $PMLCOMB bpm $PMLCOMBMASK add+ ver-" >> $CLCOM
  if [ $AUTOMKDISP -lt 1 ]; then
    echo "display $PMLCOMB 1 fi+" >> $CLCOM
  fi
  echo "logout" >> $CLCOM

#cat $CLCOM

  cat $CLCOM | cl >& $TMPLOG
  cat $TMPLOG >> $CLLOG
  ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`

  if [ $ERRSTAT -ne 0 ]; then
   echo "# Error in IRAF ... Exiting #"
   echo "# Check $CLLOG. #" 
   exit 1
  fi
  echo $PMLCOMB >> $MLCOMBTBL
  i=`expr $i + 1`
done
echo tmp.offset >> $TMPFILELIST



echo "### Pattern Matching of Coadded Frames ###"
echo "### Pattern Matching of Coadded Frames ###" >> $CLLOG 
echo "# Calculate Sigma for Single Frame #"
echo "# Calculate Sigma for Single Frame #" >> $CLLOG 
echo "del tmp.sigma" > $CLCOM 
echo "imstatrej `sed -n 1p $MLCOMBTBL`[300:600,300:600] 3 > tmp.sigma" >> $CLCOM
echo "logout" >> $CLCOM

cat $CLCOM | cl >& $TMPLOG
cat $TMPLOG >> $CLLOG
ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`

if [ $ERRSTAT -ne 0 ]; then
 echo "# Error in IRAF ... Exiting #"
 echo "# Check $CLLOG. #" 
 exit 1
fi
SIGMA=`gawk '{if(NR==1)print($3*1)}' tmp.sigma` 
echo "  sigma=$SIGMA" 

echo " # Object Detection #" 
echo " # Object Detection #" >> $CLLOG 
echo "noao" > $CLCOM
echo "digiphot" >> $CLCOM
echo "apphot" >> $CLCOM
for filename in `cat $MLCOMBTBL`; do  
  COORDFILE=`echo $filename | sed 's/.imh/.coo/g' | sed 's/.fits/.coo/g'`  
  echo $COORDFILE >> $GLOBALTMPFILELIST
  if [ -f $COORDFILE ]; then
    \rm $COORDFILE 
  fi
cat <<EOF >> $CLCOM
  imdel tmpmed
  median "$filename" tmpmed 5 5
  imshift tmpmed tmpmed -20 -20 boundar=constant
  imshift tmpmed tmpmed 40 40 boundar=constant
  imshift tmpmed tmpmed -20 -20 boundar=constant
  daofind tmpmed output=$COORDFILE fwhmpsf=8 thresho=4 verify- sigma=$SIGMA sharplo=-0.5 sharphi=5 roundlo=-5 roundhi=5
  imdel tmpmed
EOF
  if [ $AUTOMKDISP -lt 2 ]; then
    echo "display $filename 1 fi+" >> $CLCOM
    echo "tvmark 1 $COORDFILE number+ txsize=2" >> $CLCOM
  fi
done
echo "logout" >> $CLCOM 

cat $CLCOM | cl >& $TMPLOG
cat $TMPLOG >> $CLLOG
ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`

if [ $ERRSTAT -ne 0 ]; then
 echo "# Error in IRAF ... Exiting #"
 echo "# Check $CLLOG. #" 
 exit 1
fi

i=1 
for filename in `cat $MLCOMBTBL`; do  
  COORDFILE=`echo $filename | sed 's/.imh/.coo/g'| sed 's/.fits/.coo/g'`  
  COORDFILE2=`echo $filename | sed 's/.imh/2.coo/g' | sed 's/.fits/2.coo/g'`  
  XOFFSET=`sed -n ${i}p $OFFSETTABLE | gawk '{print($1)}'`
  YOFFSET=`sed -n ${i}p $OFFSETTABLE | gawk '{print($2)}'`
  grep -v "^#" $COORDFILE | gawk -v dx=$XOFFSET -v dy=$YOFFSET '{print($1+dx,$2+dy)}' > $COORDFILE2
  echo $COORDFILE >> $GLOBALTMPFILELIST
  echo $COORDFILE2 >> $GLOBALTMPFILELIST
  i=`expr $i + 1`
done

echo "# Pattern Matching #"
echo "# Pattern Matching #" >> $CLLOG
i=1 
for filename in `cat $MLCOMBTBL`; do   
  COORDFILE2=`echo $filename | sed 's/.imh/2.coo/g' | sed 's/.fits/2.coo/g'`  
  echo $COORDFILE2 >> $GLOBALTMPFILELIST
#  echo $COORDFILE2 $i
  if [ $i -eq 1 ]; then
    BASEFILE=$COORDFILE2
    echo "0 0" > tmp.offset
  else
    (./matchpat $BASEFILE $COORDFILE2 20 >> tmp.offset) >& $TMPLOG
    cat $TMPLOG
    cat $TMPLOG >> $CLLOG
  fi
  i=`expr $i + 1`
done
#paste $OFFSETTABLE tmp.offset | gawk '{print($1-$3,$2-$4)}' > tmp.offset2
paste $OFFSETTABLE tmp.offset | gawk '{print($1+$3,$2+$4)}' > tmp.offset2
echo tmp.offset2 >> $TMPFILELIST

echo "### Shift and Add ###"
echo "### Shift and Add ###" >> $CLLOG
echo "imdel tmp.tot.imh" > $CLCOM
echo "imcombine @$MLCOMBTBL tmp.tot combine=median reject=sigclip outtype=real lsigma=2 hsigma=2 offset=tmp.offset2 masktype=goodvalue maskvalue=0" >> $CLCOM
if [ $AUTOMKDISP -lt 2 ]; then
  echo "display tmp.tot 1 fi+" >> $CLCOM
fi
echo "imdel RESULTS/$OUTIM" >> $CLCOM 
echo "imcopy tmp.tot RESULTS/$OUTIM" >> $CLCOM 
echo "imdel tmp.tot.imh" >> $CLCOM
#if( $AUTOMKFRINGE == 1 && $NPOINT > 5 ) then
#echo "imcopy $FRINGEFILE RESULTS/$FRINGEOUTIM" >> $CLCOM 
#endif
echo "logout" >> $CLCOM

cat $CLCOM | cl >& $TMPLOG
cat $TMPLOG >> $CLLOG
ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`

if [ $ERRSTAT -ne 0 ]; then
 echo "# Error in IRAF ... Exiting #"
 echo "# Check $CLLOG. #" 
 exit 1
fi
\cp tmp.offset2 $OFFSETTABLE

echo "### $0 Finished ###"
echo "### $0 Finished ###" >> $CLLOG

echo "Final frame is saved as RESULTS/"$HEADER".fits"
echo "Single frames are saved as;"
cat $MLCOMBTBL
echo "Final frame is saved as RESULTS/"$HEADER".fits" >> $CLLOG
echo "Single frames are saved as;" >> $CLLOG
cat $MLCOMBTBL >> $CLLOG

echo "FRINGE File is saved as $FRINGEFILE"
echo "FRINGE File is saved as $FRINGEFILE" >> $CLLOG

echo "Offset pattern file is saved as "$OFFSETTABLE
echo "Offset pattern is : " 
cat $OFFSETTABLE
echo "Offset pattern file is saved as "$OFFSETTABLE >> $CLLOG
echo "Offset pattern is : "  >> $CLLOG
cat $OFFSETTABLE >> $CLLOG

echo "IRAF log written in $CLLOG"

if [ $AUTOMKDEBUG -eq 1 ]; then
echo "### Erase Temporary Files ###"
echo "### Erase Temporary Files ###" >> $CLLOG
  for filename in `cat $TMPFILELIST`; do
    if [ -f $filename ]; then
      \rm $filename
    fi
  done
  \rm $TMPFILELIST
fi

#END time
if [ $ARCH = "Linux" ]; then
    SECEND=`date +%s` 
else
    SECEND=`date +"%d %H %M %S"| gawk '{print($1*24*3600+$2*3600+$3*60+$4)}'` 
fi

echo $SECSTART $SECEND | gawk '{printf("Took %d sec to complete\n",$2-$1)}' >> $CLLOG
echo $SECSTART $SECEND | gawk '{printf("Took %d sec to complete\n",$2-$1)}'
